﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace GeometricObjects
{
    public abstract class GeometricObject : IComparable, INotifyPropertyChanged
  {
    // Feld
    protected Point _Center = new Point();

    // Ereignisse
    public event MovingEventHandler Moving;
    public event MovedEventHandler Moved;
    public event InvalidMeasureEventHandler InvalidMeasure;
    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged(string propertyName)
    {
        if (PropertyChanged != null)
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    protected virtual void OnMoving(MovingEventArgs e)
    {
      if (Moving != null)
        Moving(this, e);
    }

    protected virtual void OnMoved(EventArgs e)
    {
      if (Moved != null)
        Moved(this, e);
    }

    protected void OnInvalidMeasure(InvalidMeasureEventArgs e)
    {
      if (InvalidMeasure != null)
        InvalidMeasure(this, e);
      else
        throw e.Error;
    }

    // Statische Eigenschaft
    protected static int _CountGeometricObjects;
    public static int CountGeometricObjects
    {
      get { return _CountGeometricObjects; }
    }

    // Konstruktor
    protected GeometricObject()
    {
      GeometricObject._CountGeometricObjects++;
    }

    // Eigenschaften
    public virtual double XCoordinate
    {
        get { return _Center.X; }
        set
        {
            _Center.X = value;
            OnPropertyChanged("XCoordinate");
        }
    }

    public virtual double YCoordinate
    {
        get { return _Center.Y; }
        set
        {
            _Center.Y = value;
            OnPropertyChanged("YCoordinate");
        }
    }

    // Abstrakte Methoden
    public abstract double GetArea();
    public abstract double GetCircumference();

    // Instanzmethoden
    public virtual int CompareTo(Object @object)
    {
      GeometricObject geoObject = @object as GeometricObject;
      if (geoObject != null)
      {
        if (GetArea() < geoObject.GetArea()) return -1;
        if (GetArea() == geoObject.GetArea()) return 0;
        return 1;
      }
      throw new ArgumentException("Ungültige Objektübergabe. Es wird der Typ 'GeometricObject' erwartet.");
    }

    public virtual void Move(double dx, double dy)
    {
      // Moving-Ereignis
      MovingEventArgs e = new MovingEventArgs();
      OnMoving(e);
      if (e.Cancel == true)
        return;
      XCoordinate += dx;
      YCoordinate += dy;
      // Moved-Ereignis
      OnMoved(new EventArgs());
    }

    public virtual void Move(Point center)
    {
      _Center = center;
    }

    // Klassenmethode
    public static int Bigger(GeometricObject object1, GeometricObject object2)
    {
        if (object1 == null || object2 == null) return 0;
        if (object1 == null) return -1;
        if (object2 == null) return 1;
        if (object1.GetArea() > object2.GetArea()) return 1;
        if (object1.GetArea() < object2.GetArea()) return -1;
        return 0;
    }
  }
}    
